//+------------------------------------------------------------------+
//|                                                MQL4OrderPool.mqh |
//|                                                      nicholishen |
//|                            http://www.reddit.com/u/nicholishenFX |
//+------------------------------------------------------------------+
#property copyright "nicholishen"
#property link      "http://www.reddit.com/u/nicholishenFX"
#property version   "1.00"
#property strict

#define SORT_CLOSE_TIME_DESCENDING 0
#define SORT_OPEN_TIME_DESCENDING 1
#define SORT_PROFIT_DESCENDING 2
#define SORT_NETPROFIT_DESCENDING 3
#define SORT_LOTS_DESCENDING 4
#define SORT_POINTS_DESCENDING 5
#define SORT_SYMBOL 6
#include <Arrays\ArrayObj.mqh>

enum ORDER_TYPES
{
   ORDERS_PENDING,
   ORDERS_FILLED,
   ORDERS_ALL   
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class MQL4Order : public CObject
{
protected:
   int            m_ticket;
   int            m_magic;
   datetime       m_open_time;
   datetime       m_close_time;
   double         m_open_price;
   double         m_close_price;
   double         m_lots;
   double         m_profit;
   double         m_commish;
   double         m_swap;
   int            m_type;
   double         m_tp;
   double         m_sl;
   string         m_comment;
public:
   bool           Init(const int ticket);
   double         OrderPoints()       const;
   int            OrderTicket()       const { return m_ticket;                    }
   int            OrderMagicNumber()  const { return m_magic;                     }
   datetime       OrderOpenTime()     const { return m_open_time;                 }
   datetime       OrderCloseTime()    const { return m_close_time;                }
   double         OrderOpenPrice()    const { return m_open_price;                }
   double         OrderClosePrice()   const { return m_close_price;               }
   double         OrderLots()         const { return m_lots;                      }
   double         OrderProfit()       const { return m_profit;                    }
   double         OrderProfitNet()    const { return NormalizeDouble(m_profit+m_swap+m_commish,2);   }
   double         OrderCommission()   const { return m_commish;                   }
   double         OrderSwap()         const { return m_swap;                      }
   int            OrderType()         const { return m_type;                      }
   double         OrderTakeProfit()   const { return m_tp;                        }
   double         OrderStopLoss()     const { return m_sl;                        }
   string         OrderComment()      const { return m_comment;                   }
   void           OrderPrint()        const { if(Select()) ::OrderPrint();        }
   virtual int    Compare(const CObject *node,const int mode=0)const;
protected:
   bool           Select() const{ return OrderSelect(m_ticket,SELECT_BY_TICKET);}
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class MQL4OrderPool : public CObject
{
protected:
   int            m_index;
   CArrayObj      m_arr;
  
   int            m_cnt;
   string         m_symbol;
   int            m_magic;
   int            m_mode;
   ORDER_TYPES    m_types;
   int            m_sort;
public:
                  MQL4OrderPool();
                 ~MQL4OrderPool();  
   MQL4Order*     operator[](const int index)const{return (MQL4Order*)m_arr.At(index);}
   int            Total() const { return m_arr.Total();}
   void           Sort(const int mode=0) { m_sort=mode; m_arr.Sort(mode);}
   void           Refresh();
   bool           Init( string       symbol = NULL,
                        const int    magic  = 0,
                        const int    mode   = MODE_TRADES,
                        const ORDER_TYPES types = ORDERS_ALL,
                        const int    sort_mode = SORT_CLOSE_TIME_DESCENDING);
   MQL4Order*     GetFirstNode()    { m_index = 0; return this[0];}
   MQL4Order*     GetLastNode()     { m_index = 0; return this[Total()-1];}
   MQL4Order*     GetNextNode()     { return this[++m_index];}
   MQL4Order*     GetPrevNode()     { return this[--m_index];}
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
MQL4OrderPool::MQL4OrderPool()
{
}
//+------------------------------------------------------------------+
MQL4OrderPool::~MQL4OrderPool()
{
}
//+------------------------------------------------------------------+
bool MQL4OrderPool::Init(  string       symbol = NULL,
                           const int    magic  = 0,
                           const int    mode   = MODE_TRADES,
                           const ORDER_TYPES types = ORDERS_ALL,
                           const int    sort_mode = SORT_CLOSE_TIME_DESCENDING)
{
   m_index     = 0;
   m_cnt       = 0;
   m_symbol    = symbol;
   m_magic     = magic;
   m_mode      = mode;
   m_types     = types;
   m_sort      = sort_mode;
   Refresh(); 
   return true;
}  
//+------------------------------------------------------------------+
void MQL4OrderPool::Refresh(void)
{
   if(m_mode==MODE_TRADES && OrdersTotal()==m_cnt)
      return;
   if(m_mode==MODE_HISTORY && OrdersHistoryTotal()==m_cnt)
      return;
   m_arr.Clear();
   m_arr.Reserve((int)MathMax((double)OrdersTotal(),(double)OrdersHistoryTotal()));
   int total = m_mode==MODE_TRADES ? OrdersTotal() : OrdersHistoryTotal();
   for(int i=0;i<total;i++)
   {
      if((OrderSelect(i,SELECT_BY_POS,m_mode))       &&
         (m_symbol==NULL || m_symbol == OrderSymbol()) &&
         (m_magic==0     || m_magic == OrderMagicNumber())&&
         (m_types==ORDERS_ALL||(m_types==ORDERS_FILLED && OrderType()<=1)||(m_types==ORDERS_PENDING && OrderType()>1))
        )
      {
         MQL4Order *order = new MQL4Order;
         if(!order.Init(OrderTicket()))
            delete order;   
         if(order != NULL)
            m_arr.Add(order);
      }
   }
   m_arr.Sort(m_sort);
   m_cnt = m_arr.Total();
}
//+------------------------------------------------------------------+
//|      MQL4Order src                                               |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|      MQL4Order src                                               |
//+------------------------------------------------------------------+
double MQL4Order::OrderPoints(void)const
{
   if(Type()==OP_BUY)
      return this.OrderClosePrice()-this.OrderOpenPrice();
   if(Type()==OP_SELL)
      return this.OrderOpenPrice()-this.OrderClosePrice();
   return 0;
}
//+------------------------------------------------------------------+
bool MQL4Order::Init(const int ticket)
{
   if(!OrderSelect(ticket,SELECT_BY_TICKET))
      return false;
   m_ticket       = ::OrderTicket();
   m_magic        = ::OrderMagicNumber();
   m_open_time    = ::OrderOpenTime();
   m_close_time   = ::OrderCloseTime();
   m_open_price   = ::OrderOpenPrice();
   m_close_price  = ::OrderClosePrice();
   m_lots         = ::OrderLots();
   m_profit       = ::OrderProfit();
   m_commish      = ::OrderCommission();
   m_swap         = ::OrderSwap();
   m_type         = ::OrderType();
   m_tp           = ::OrderTakeProfit();
   m_sl           = ::OrderStopLoss();
   m_comment      = ::OrderComment();
   return true;
}
//+------------------------------------------------------------------+
int MQL4Order::Compare(const CObject *node,const int mode=0)const
{
   MQL4Order *other = (MQL4Order*)node;
   if(mode==SORT_PROFIT_DESCENDING)
   {
      if(this.OrderProfit()>other.OrderProfit()) return -1;
      if(this.OrderProfit()<other.OrderProfit()) return 1;
      return 0;
   }
   if(mode==SORT_LOTS_DESCENDING)
   {
      if(this.OrderLots()>other.OrderLots()) return -1;
      if(this.OrderLots()<other.OrderLots()) return 1;
      return 0;
   }
   if(mode==SORT_NETPROFIT_DESCENDING)
   {
      if(this.OrderProfitNet()>other.OrderProfitNet()) return -1;
      if(this.OrderProfitNet()<other.OrderProfitNet()) return 1;
      return 0;
   }
   if(mode==SORT_POINTS_DESCENDING)
   {
      if(this.OrderPoints()>other.OrderPoints()) return -1;
      if(this.OrderPoints()<other.OrderPoints()) return 1;
      return 0;
   }
   if(mode==SORT_OPEN_TIME_DESCENDING)
   {
      if(this.OrderOpenTime()>other.OrderOpenTime()) return -1;
      if(this.OrderOpenTime()<other.OrderOpenTime()) return 1;
      return 0;
   }
   if(this.OrderCloseTime()>other.OrderCloseTime()) return -1;
   if(this.OrderCloseTime()<other.OrderCloseTime()) return 1;
   return 0;
}
//+------------------------------------------------------------------+